import { validateParams } from "../../lib/core"
import layer from "../layer/index"
import { Style, Fill } from 'ol/style' // Stroke, Fill, Circle,
import { Vector as VectorSource } from 'ol/source';
import { Vector as VectorLayer } from 'ol/layer';
import { Draw, Modify, Snap } from 'ol/interaction';
import { Polygon, MultiPolygon } from 'ol/geom';
import Feature from 'ol/Feature'
import {
    unByKey
} from 'ol/Observable.js'
import { intersect, polygon as turfPolygon } from '@turf/turf'
import Translate from "ol/interaction/Translate";
/**
 * 影响区 和对比区绘制类
 */

const drawSourceLayerKey = "DRAW_SOURCE_LAYER"

export const DRAW_TYPE = {
    INFLUENCE_AREA: 'INFLUENCE_AREA', // 影响区
    COMPARISON_AREA: 'COMPARISON_AREA'// 影响对比区
}

export const FILL_COLOR = {
    INFLUENCE_AREA: '#02ffff', // 影响区
    COMPARISON_AREA: '#307cee'// 影响对比区
}


/**
 * 影响区，手动绘制 
 * 对比区，手动绘制，自动绘制(拷贝手动绘制，做一定偏移)
 */
class InfluenceCompasionDraw {
    constructor (params) {
        validateParams(['map'], params)
        this.drawTool = null;
        this.snapTool = null;
        this.source = null;
        this.modifyTool = null;
        this.drawOptions = null;
        this.drawstart = params.drawstart;
        this.drawend = params.drawend;
        this.drawType = DRAW_TYPE.INFLUENCE; // 默认绘制影响区
        this.measure = params.measure
        this.drawOptions = params.drawOptions || {}
        this.listeners = []
        this.map = params.map;
        this.fillColor = FILL_COLOR || params.fillColor;
        this.bindEscEventListerner = this.escEventListerner.bind(this)
        this.clipGeometryJson = params.clipGeometryJson // 剪切 json
    }

    unListeners () {
        this.listeners.forEach(listener => {
            unByKey(listener)
        })
    }

    escEventListerner (e) {
        if (e.code === 'Escape') {
            this.complete(this.drawType)
        }
    }

    start (key) {
        /**
         * 绘制前销毁之前的工具
         */
        this.destory()
        /**
         * 绘制前清理历史图层 ， 如果需要保留可以注释该代码
         */
        this.clear(key)
        this.drawType = key
        this.addDrawLayer()
        this.drawTool = new Draw({
            freehand: true,
            source: this.source,
            type: 'Polygon',
            ...this.drawOptions
        });
        this.map.addInteraction(this.drawTool);
        this.snapTool = new Snap({ source: this.source });
        this.map.addInteraction(this.snapTool);
        this.modifyTool = new Modify({ source: this.source });
        this.map.addInteraction(this.modifyTool);
        // 添加Translate互动选项以实现拖动功能
        const translate = new Translate({
            layers: [this.vectorLayer],
            hitTolerance: 0
        });
        this.map.addInteraction(translate);

        let listener = this.drawTool.on('drawstart', (evt) => {
            this.drawstart && this.drawstart(evt)
        })
        this.listeners.push(listener)
        listener = this.drawTool.on('drawend', (evt) => {
            this.drawend && this.drawend(evt)
        })
        document.addEventListener('keydown', this.bindEscEventListerner)
        this.listeners.push(listener)
    }

    complete (type) {
        // 如果需要结束绘制的不是 当前正在绘制的,那么不执行任何操作
        if (type !== this.drawType) {
            return;
        }
        if (!this.drawTool) {
            return
        }
        document.removeEventListener('keydown', this.bindEscEventListerner)
        /**
         * 判断是否绘制了要素，如果绘制了，那么就把图层添加到地图上, 如果没有要素,就不必再添加了
         */
        let drawFeatures = this.vectorLayer.getSource().getFeatures()

        if (drawFeatures.length) {
            this.addAreaLayer(drawFeatures)
        }
        this.destory()
    }

    /**
     *  添加绘制后的 图层到地图
     * @param {*} coordinates 
     * @returns 
     */
    addAreaLayer (drawFeatures) {
        const projectionCode = this.map.getView().getProjection().getCode()
        let features = []
        console.log('drawFeatures', drawFeatures)
        drawFeatures.forEach(feature => {
            feature = feature.clone()
            let coordinates = feature.getGeometry().transform(projectionCode, 'EPSG:4326').getCoordinates()
            let intersectPolygons = this.clip(coordinates)
            intersectPolygons.forEach(intersectPolygon => {
                let circleFeature = null
                if (intersectPolygon.geometry.type === "Polygon") {
                    circleFeature = new Feature({
                        geometry: new Polygon(intersectPolygon.geometry.coordinates).transform('EPSG:4326', projectionCode),
                    })
                }
                if (intersectPolygon.geometry.type === "MultiPolygon") {
                    circleFeature = new Feature({
                        geometry: new MultiPolygon(intersectPolygon.geometry.coordinates).transform('EPSG:4326', projectionCode),
                    })
                }
                features.push(circleFeature)
            })
        })
        if (!features.length) {
            return
        }
        const vectorSource = new VectorSource({
            features: features,
        })
        let vectorLayer = new VectorLayer({
            name: this.drawType,
            source: vectorSource,
            zIndex: 1,
            opacity: 0.6
        })
        let color = this.fillColor[this.drawType]
        vectorLayer.setStyle(() => {
            return new Style({
                fill: new Fill({
                    color: color,
                })
            })
        })
        this.map.addLayer(vectorLayer)
    }

    // 要素裁剪

    clip (polygons) {
        if (!this.clipGeometryJson) {
            return polygons
        }
        const intersectPolygons = []
        if (polygons.length === 1) {
            polygons = [polygons]
        }
        polygons.forEach(polygon => {
            this.clipGeometryJson.geometries.forEach(geometry => {
                geometry.coordinates.forEach(coordinate => {
                    let result = intersect(turfPolygon(coordinate), turfPolygon(polygon))
                    if (result) {
                        intersectPolygons.push(result)
                    }
                })
            })
        })

        return intersectPolygons
        // var intersection = turf.intersect(poly1, poly2);
    }

    addDrawLayer () {
        this.source = new VectorSource({ wrapX: false });
        this.vectorLayer = new VectorLayer({
            name: drawSourceLayerKey,
            source: this.source
        });

        this.map.addLayer(this.vectorLayer)
    }

    destory () {
        if (this.drawTool) {
            this.map.removeInteraction(this.drawTool);
            this.unListeners()
        }
        if (this.vectorLayer) {
            this.source.clear()
            this.map.removeLayer(this.vectorLayer)
        }
        this.snapTool && this.map.removeInteraction(this.snapTool);
        this.modifyTool && this.map.removeInteraction(this.modifyTool);

        this.snapTool = null
        this.modifyTool = null
        this.drawTool = null
        this.vectorLayer = null
        this.source = null

    }

    auto (option = { offset: 0.01 }) {
        this.clear(DRAW_TYPE.COMPARISON_AREA)
        const layers = layer.getLayers({
            map: this.map,
            codeKey: 'name',
            codeValue: DRAW_TYPE.INFLUENCE_AREA
        })
        if (!layers.length) {
            throw new Error('请先绘制影响区')
        }
        console.log('layers', layers)
        this.start(DRAW_TYPE.COMPARISON_AREA);
        let features = []
        const projectionCode = this.map.getView().getProjection().getCode()
        layers.forEach(layer => {
            let tempFeatures = layer.getSource().getFeatures().map(feature => {
                feature = feature.clone()
                let geometry = feature.getGeometry().transform(projectionCode, 'EPSG:4326')
                console.log('getGeometry', option)
                // console.log('getGeometry', getGeometry)
                geometry.translate(option.offset, option.offset)
                console.log('getGeometry', geometry)
                geometry.transform('EPSG:4326', projectionCode)
                // console.log('getGeometry', geometry)
                // feature.setGeometry(geometry)
                return feature
            })
            features = [...features, ...tempFeatures]
        })
        console.log('features', features)
        this.source.addFeatures(features)
        // this.vectorLayer.update()
    }

    clear (key) {
        // 如果需要结束绘制的不是 当前正在绘制的,那么不执行任何操作
        if (this.vectorLayer && key === this.drawType) {
            this.vectorLayer.getSource().clear()
        }
        layer.removeLayers({
            map: this.map,
            codeKey: 'name',
            codeValue: key
        })
    }
}

export default InfluenceCompasionDraw