/*
 * @Description: 绘制相关的代码
 * @Author: 王强
 * @Date: 2022-09-08 22:02:48
 * @LastEditTime: 2022-09-17 11:22:14
 * @LastEditors: 王强
 */
import Overlay from 'ol/Overlay'
import { Vector as VectorSource } from 'ol/source';
import { Vector as VectorLayer } from 'ol/layer';
import { Draw, Modify, Snap } from 'ol/interaction';
import { transform } from "ol/proj"
import { validateParams, getCenterFromCoordinates } from "../../lib/core"
import {
    unByKey
} from 'ol/Observable.js'
import {
    LineString,
    Polygon
} from 'ol/geom.js'
import { measureArea, measureLength } from "../measure/index"
import "./measure.css"
export const DRAW_TYPE = {
    POINT: 'Point',
    CIRCLE: 'Circle',
    LineString: 'LineString',
    PLONY_GON: 'Polygon'
}
const drawSourceLayerKey = "DRAW_SOURCE_LAYER"
class DrawTool {


    constructor (params) {
        validateParams(['map'], params)
        this.map = null;
        this.measure = null;// 是否开启测量
        this.drawTool = null;
        this.snapToo = null;
        this.modifyTool = null;
        this.source = new VectorSource({ wrapX: false });
        this.vectorLayer = null;
        this.layerName_ = 'DRAW_SOURCE_LAYER'
        this.drawOptions = null;
        this.drawType = null;
        this.drawstart = null;
        this.drawend = null;
        this.measureTooltipElement = null;
        this.measureTooltip = null;
        this.drawType = params.type;
        this.measure = params.measure
        if (params.measure) {
            if (!['LineString', 'Polygon'].includes(this.drawType)) {
                throw new Error("只能测量 'LineString', 'Polygon'")
            }
        }

        if (params.measure) {
            let listener;
            this.drawstart = (evt) => {
                this.createTooltip()
                params.drawstart && params.drawstart(evt)
                const sketch = evt.feature
                let coordinate = evt.coordinate
                listener = sketch.getGeometry().on('change', (event) => {
                    const geom = event.target
                    let output
                    if (geom instanceof Polygon) {
                        output = measureArea(this.map, geom)
                        coordinate = getCenterFromCoordinates(geom.clone()
                            .transform("EPSG:3857", "EPSG:4326")
                            .getCoordinates()[0])
                        coordinate = transform(coordinate, 'EPSG:4326', 'EPSG:3857')
                    } else if (geom instanceof LineString) {
                        output = measureLength(this.map, geom)
                        coordinate = getCenterFromCoordinates(geom.clone()
                            .transform("EPSG:3857", "EPSG:4326")
                            .getCoordinates())
                        coordinate = transform(coordinate, 'EPSG:4326', 'EPSG:3857')
                    }
                    this.measureTooltipElement.innerHTML = output.value + output.unit
                    this.measureTooltip.setPosition(coordinate)
                })

            }
            // const featureGeoJson = JSON.parse(new GeoJSON().writeFeature(evt.feature))
            this.drawend = (evt) => {
                params.drawend && params.drawend(evt)
                const feature = evt.feature
                let measureTooltipCopy = this.cloneTooltip()
                this.removeTooltip()
                this.measureTooltip = null
                const closeDom = document.createElement('span')
                closeDom.innerHTML = 'x'
                closeDom.className = 'close'
                closeDom.onclick = () => {
                    this.vectorLayer.getSource().removeFeature(feature)
                    let element = measureTooltipCopy && measureTooltipCopy.getElement()
                    if (element && element.parentNode) {
                        element.parentNode.removeChild(element)
                    }
                    console.log('measureTooltipCopy', measureTooltipCopy)
                    measureTooltipCopy && this.map.removeOverlay(measureTooltipCopy)
                }
                measureTooltipCopy && measureTooltipCopy.getElement().appendChild(closeDom)
                unByKey(listener)
            }
        } else {
            this.drawstart = params.drawstart
            this.drawend = params.drawend
        }
        this.drawOptions = params.drawOptions || {}
        this.vectorLayer = new VectorLayer({
            name: drawSourceLayerKey,
            source: this.source
        });
        this.map = params.map;
        this.map.addLayer(this.vectorLayer)
        this.initDraw()
    }

    initDraw () {
        this.drawTool = new Draw({
            source: this.source,
            type: this.drawType || DRAW_TYPE.POINT,
            ...this.drawOptions
        });

        this.map.addInteraction(this.drawTool);
        this.snapTool = new Snap({ source: this.source });
        this.map.addInteraction(this.snapTool);
        this.modifyTool = new Modify({ source: this.source });
        this.map.addInteraction(this.modifyTool);

        if (this.drawstart) {
            this.drawTool.on('drawstart', (evt) => {
                this.drawstart(evt)
            })
        }
        if (this.drawend) {
            this.drawTool.on('drawend', (evt) => {
                this.drawend(evt)
            })
        }
    }

    unDraw () {
        if (this.drawTool) {
            this.map.removeInteraction(this.drawTool);
            !(this.drawTool.sketchFeature_) && this.removeTooltip()
        }
        this.snapTool && this.map.removeInteraction(this.snapTool);
        this.modifyTool && this.map.removeInteraction(this.modifyTool)
    }

    setType (type) {
        if (this.measure) {
            if (!['LineString', 'Polygon'].includes(type)) {
                throw new Error("只能测量 'LineString', 'Polygon'")
            }
        }
        this.drawType = type
        this.unDraw()
        this.initDraw()
    }
    setDrawStyle (style) {
        this.unDraw()
        this.drawOptions.style = style
        this.initDraw()
    }

    undo () {
        if (this.drawTool) {
            this.drawTool.removeLastPoint()
            !(this.drawTool.sketchFeature_) && this.removeTooltip()
        }
    }

    clear () {
        this.source && this.source.clear()
        const overLayers = this.map.getOverlays().getArray()
        for (let i = 0; i < overLayers.length; i++) {
            const overlay = overLayers[i]
            if (overlay.get("overLayId") === 'DRAW_OVER_LAYER') {
                i = i - 1
                this.map.removeOverlay(overlay);
            }
        }
    }

    removeTooltip () {
        if (this.measureTooltipElement && this.measureTooltipElement.parentNode) {
            this.measureTooltipElement.parentNode.removeChild(this.measureTooltipElement)
            this.measureTooltipElement = null
        }
        this.measureTooltip && this.map.removeOverlay(this.measureTooltip)
    }
    createTooltip () {
        this.measureTooltipElement = document.createElement('div')
        this.measureTooltipElement.className = 'map-measure tooltip'
        this.measureTooltip = new Overlay({
            element: this.measureTooltipElement,
            positioning: 'center-center'
        })
        this.map.addOverlay(this.measureTooltip)
    }

    cloneTooltip () {
        if (!this.measureTooltipElement) {
            return
        }
        const element = this.measureTooltipElement.cloneNode(true)
        const overlay = new Overlay({
            element: element,
            positioning: 'center-center'
        })
        overlay.setPosition(this.measureTooltip.getPosition())
        overlay.set("overLayId", 'DRAW_OVER_LAYER')
        this.map.addOverlay(overlay)
        return overlay
    }
}

export default DrawTool